////////////////////////////////////////////////////////////////////////////
// Aldo Hoeben / fieldOfView.com
// Copyright 2007 Aldo Hoeben
//
/////////////////////////////////////////////////////////////////////////////

var map = null;
var marker = null;
var geocoder = null;

var apply_control = null;

var search_match = /doc_id=(\d*)/.exec(document.location.search);
var doc_id = (search_match != null) ? search_match[1] : 0;


/*
 Google Maps initialisation
 */
function initialize() {
	if (GBrowserIsCompatible()) {
		// initialise Google Map
		var map_div = document.getElementById("map_canvas");
		map = new GMap2(map_div);
		map.disableDoubleClickZoom();
		map.setCenter(new GLatLng(30, -20), 2);

		// initialise geocoder
		geocoder = new GClientGeocoder();

		map.addControl(new ControlsBar());
		apply_control = new ApplyControl();

		// hide controls when map not active
		map.hideControls();

		GEvent.addListener(map, "mouseover", function(){
			map.showControls();
		});
		GEvent.addListener(map, "mouseout", function(){
			map.hideControls(); 
		});

		GEvent.addListener(map, "dblclick", function(overlay,point) {
			setMarker(point);
		});
	}
}

/*
 Custom controls for our Google Map to suite the Bridge UI:
  - Zoom in/out
  - Geocode form
  - Maptype toggle (map/sat/hybrid)
 */
function ControlsBar() {
}
ControlsBar.prototype = new GControl();

// Creates a DIV for each of the buttons and places them in a container
// DIV which is returned as our control element. We add the control to
// to the map container and return the element for the map class to
// position properly.
ControlsBar.prototype.initialize = function(map) {
	var container = document.createElement("div");
	container.style.background = "url(control-bg.png) repeat-x";
	container.style.display = "block";
	container.style.height = "25px";
	container.style.width = "100%";
	container.style.padding = "5px"

	// zoom in button
	var zoom_in = this._createButtonDiv(container);
	zoom_in.style.background = "url(zoomin.png)";
	GEvent.addDomListener(zoom_in, "click", function() {
		map.zoomIn();
	});

	// zoom out button
	var zoom_out = this._createButtonDiv(container);
	zoom_out.style.backgroundImage = "url(zoomout.png)";
	zoom_out.style.paddingRight = "3px";
	zoom_out.style.borderRight = "1px solid #454545";
	GEvent.addDomListener(zoom_out, "click", function() {
		map.zoomOut();
	});

	// geocode button
	var geocode_button = this._createButtonDiv(container);
	geocode_button.style.backgroundImage = "url(geocode.png)";
	geocode_button.style.paddingRight = "3px";
	geocode_button.style.borderRight = "1px solid #454545";

	// add geocode form
	/* note: Opera does not cope with dynamically generated form tags
	 * that try to do onsubmit="return false"
	 */
	//var geocode_form = document.createElement("form");
	var geocode_form = document.getElementById("geocode_form");
	geocode_form.style.marginTop = "27px";
	geocode_form.style.display = "none";
	var geocode_input = document.createElement("input");
	geocode_input.type = "text";
	geocode_form.appendChild(geocode_input);
	geocode_button.appendChild(geocode_form);

	GEvent.addDomListener(geocode_form, "submit", function() {
		var address = this.childNodes[0].value;
		if (geocoder) {
			geocoder.getLatLng(
				address,
				function(point) {
					if (!point) {
						alert(address + " not found");
					} else {
						setMarker(point)
					}
				}
			);
		}
		this.style.display = "none";

		return false;
	});

	GEvent.addDomListener(geocode_form, "dblclick", function() {
		this.childNodes[0].value = "";
	});
	GEvent.addDomListener(geocode_button, "click", function(event) {
		if(event.clientY < 30) geocode_form.style.display = (geocode_form.style.display == "none") ? "block" : "none";
	});
	GEvent.addDomListener(map, "click", function(event) {
		geocode_form.style.display = "none";
	});

	// map/sat/hybrid button
	var maptype_icons = ["map.png","sat.png","hybrid.png"];
	var maptypes = map.getMapTypes();
	var current_maptype = -0;

	var maptype_button = this._createButtonDiv(container);
	maptype_button.style.backgroundImage = "url("+maptype_icons[current_maptype]+")";

	GEvent.addDomListener(maptype_button, "click", function() {
		current_maptype++;
		if(current_maptype >= maptypes.length) {
			current_maptype = 0;
		}
		maptype_button.style.backgroundImage = "url("+maptype_icons[current_maptype]+")";
		map.setMapType(maptypes[current_maptype]);
	});

	map.getContainer().appendChild(container);
	return container;
}

// By default, the control will appear in the top left corner of the map.
ControlsBar.prototype.getDefaultPosition = function() {
	return new GControlPosition(G_ANCHOR_TOP_LEFT);
}

// Sets the proper CSS for the given button element.
ControlsBar.prototype._createButtonDiv = function(container) {
	button = document.createElement("div");
	button.style.cssFloat = "left";
	button.style.backgroundRepeat = "no-repeat"
	button.style.width = "20px";
	button.style.height = "20px";
	button.style.marginRight = "3px";
	button.style.cursor = "pointer";

	container.appendChild(button);
	return button;
}

function ApplyControl() {
}
ApplyControl.prototype = new GControl();

ApplyControl.prototype.initialize = function(map) {
	var button = document.createElement("input");
	button.type = "button";
	button.value = "Apply";
	button.style.fontSize = "10px";

	GEvent.addDomListener(button, "click", function() {
		if(marker) {
			point = marker.getPoint();
			callBridge("applyGPS" + point.toString() );
		} else {
			callBridge("applyGPS()");
		}
		map.removeControl(apply_control);
	});

	map.getContainer().appendChild(button);
	return button;
}
ApplyControl.prototype.getDefaultPosition = function() {
	return new GControlPosition(G_ANCHOR_TOP_RIGHT, new GSize(5, 5));
}


/*
 Called from Bridge when selections in the document change
 */
function setMap(thumb_count, dec_lat, dec_lng) {
	var point;
	if(dec_lat && dec_lng) {
		point = new GLatLng(dec_lat, dec_lng);
	}
	setMarker(point, true);
	if (thumb_count > 0) {
		map_hider.style.display = "none";
	} else {
		map_hider.style.display = "block";
	}
}

/*
 Set a marker on the specified point in the Google map
 */
function setMarker(point, clean) {
	if(marker) {
		map.removeOverlay(marker)
	}
	if(point) {
		var zoom = Math.max( 13, map.getZoom() );
		map.setCenter(point, zoom);

		marker = new GMarker(point, {draggable: true});
		map.addOverlay(marker);

		GEvent.addListener(marker, "click", function() {
			setMarker();
		});

		GEvent.addListener(marker, "dragstart", function() {
		});

		GEvent.addListener(marker, "dragend", function() {
			map.addControl(apply_control);
		});
	} else {
		marker = null;
	}

	if (!clean)
	{
		map.addControl(apply_control);
	}
}

/*
 There is no direct way for an HTML page in a tabbed palette to comunicate 
 back to Bridge. Instead we use a tiny Flash movie to communicate with a 
 second Flash movie running in a NavBar.
*/
function callBridge(call_string){
	var flash_player = document.getElementById("call_bridge")
	flash_player.callBridge(doc_id, call_string);
}