////////////////////////////////////////////////////////////////////////////
// Aldo Hoeben / fieldOfView.com
// Copyright 2007 Aldo Hoeben
//
/////////////////////////////////////////////////////////////////////////////

/*
@@@START_XML@@@
<?xml version="1.0" encoding="UTF-8"?>
<ScriptInfo xmlns:dc="http://purl.org/dc/elements/1.1/" xml:lang="en_US">
     <dc:title>Bridge Geotagger</dc:title>
     <dc:description>Allows geotagging of files using a Google Maps interface within Bridge CS 3.</dc:description>
</ScriptInfo>
@@@END_XML@@@
*/

/** 
 @fileoverview Allows geotagging of files using a Google Maps interface within Bridge CS 3.
 
 <h4>Usage</h4>
  <ul>
 <li>Run the snippet in the ExtendScript Toolkit (see Readme.txt), with Adobe Bridge CS3 as the target.
 </ul>
    
  <h4>Description</h4>

   <p>Adds a tab containing a Google Maps based Geotagging interface 
   to all Bridge browser windows.  
   
   @constructor Constructor.
 */ 
function BridgeGeotagger()
{
	/**
	 The context in which this snippet can run.
	 @type String
	*/
	this.requiredContext = "\tExecute against Bridge CS3 main engine.";
	$.level = 1; // Debugging level
	this.paletteRefs = null;
	
	/**
	  The file-system location of this script
	  @type File
	*/
	var scriptsFile = new File($.fileName);
	
	/**
	  The file-system location of the HTML resource file for the Content pane
	  @type File
	*/
	this.htmlFile = new File(scriptsFile.path + "/BridgeGeotagger.html");	
	this.flashFile = new File(scriptsFile.path + "/BridgeGeotagger.swf");
	
	this.onDocumentEvent = null;
	
	// Load the XMP Script library
	if( xmpLib == undefined ) 
	{
		if( Folder.fs == "Windows" )
		{
			var pathToLib = Folder.startup.fsName + "/AdobeXMPScript.dll";
		} 
		else 
		{
			var pathToLib = Folder.startup.fsName + "/AdobeXMPScript.framework";
		}
	
		var libfile = new File( pathToLib );
		var xmpLib = new ExternalObject("lib:" + pathToLib );
	}	
}

/**
	@return True if the snippet ran as expected, false otherwise.  
	@type Boolean
*/
BridgeGeotagger.prototype.run = function()
{
	if(!this.canRun())
	{
		return false;
	}

	this.paletteRefs = new Array();
	var _this = this;



	// Create the palette of type "web", as well as a navbar
	function addUI(doc)
	{
		var _doc = doc;
		
		// Create the palette
		var palette = new TabbedPalette(doc, 
						"Geotagging", 
						"geotaggingTab", "web", 
						"file://" + _this.htmlFile.fsName + "?doc_id=" + doc.id);
						
		_this.paletteRefs[doc.id] = palette;
		palette.visible = false;
		
		// Set up NavBar containing a Flash movie for callbacks
		var hiddenBar = doc.navbars.filesystem.bottom;
		hiddenBar.visible = true;
		hiddenBar.height = 0;
		var flashListener = hiddenBar.add("flashplayer");
		flashListener.loadMovie(_this.flashFile.fsName );
		
		flashListener.invokePlayerFunction ("listen", [doc.id]);
				
		flashListener.applyGPS = function(gps_string) 
		{
			var dec_lat, dec_lng;
			var gps = gps_string.split(",");
			
			if(gps.length == 2)
			{
				dec_lat = gps[0] * 1.0;
				dec_lng = gps[1] * 1.0;
			}
			selectionApplyGPS(_doc, dec_lat, dec_lng)
		}
	}



	// Remove reference to the palette of type "web"
	function removeUI(doc)
	{
			_this.paletteRefs[doc.id] = null;
	}




	// Respond to selection changes
	function selectionsChanged(doc)
	{
		var dec_lng, dec_lat;
		var thumb_count = 0;

		for(var i=0; i < doc.selectionsLength; i++) 
		{
			var thumb = app.document.selections[i];
	
			if(thumb.hasMetadata)
			{
				thumb_count++;
				
				// Get the metadata object - wait for  valid values
				var md = thumb.synchronousMetadata;
				
				// Get GPS values from EXIF
				var gps_lat = md.read(XMPConst.NS_EXIF,"GPSLatitude");
				var gps_lat_ref = md.read(XMPConst.NS_EXIF,"GPSLatitudeRef");				
				var gps_lng = md.read(XMPConst.NS_EXIF,"GPSLongitude");
				var gps_lng_ref = md.read(XMPConst.NS_EXIF,"GPSLongitudeRef");	
				
				if(gps_lat != undefined && gps_lng != undefined) 
				{
					dec_lat = gps2dec(gps_lat, gps_lat_ref);
					dec_lng = gps2dec(gps_lng, gps_lng_ref);
					break;
				}
			}
		}
		
		var palette = _this.paletteRefs[doc.id];
		if(palette)
		{
			palette.url = "javascript:setMap(" + thumb_count + "," + dec_lat + "," + dec_lng + ")";
		}
	}

	// apply GPS coordinates to selected thumbnails
	function selectionApplyGPS(doc, dec_lat, dec_lng)
	{
		if(dec_lng != undefined && dec_lat != undefined)
		{
			// Convert decimal latitude to GPS/EXIF format
			var gps_lat = dec2gps(dec_lat);
			var gps_lat_ref = "N";
			if (gps_lat[0]<0) 
			{
				gps_lat_ref = "S";
				//gps_lat[0] = -gps_lat[0];
			} 
		
			// Convert decimal longitude to GPS/EXIF format
			var gps_lng = dec2gps(dec_lng);
			var gps_lng_ref = "E";
			if (gps_lng[0]<0) 
			{
				gps_lng_ref = "W";
				//gps_lng[0] = -gps_lng[0];
			} 
		}

		var palette = _this.paletteRefs[doc.id];

		for(var i=0; i < doc.selectionsLength; i++) 
		{
			var thumb = app.document.selections[i];
	
			if(thumb.hasMetadata)
			{
				// Get the metadata object - wait for  valid values
				var md = thumb.synchronousMetadata;
				// Get the XMP packet as a string and create the XMPMeta object
				var xmp = new XMPMeta(md.serialize());
				
				if(dec_lng != undefined && dec_lat != undefined)
				{
					// If we got coordinates, add them to the xml packet;
					xmp.setProperty(XMPConst.NS_EXIF, "GPSVersionID","2.0.0.0");
					xmp.setProperty(XMPConst.NS_EXIF, "GPSLatitude", gps_lat);
					//xmp.setProperty(XMPConst.NS_EXIF, "GPSLatitudeRef", gps_lat_ref);
					xmp.setProperty(XMPConst.NS_EXIF, "GPSLongitude", gps_lng);			
					//xmp.setProperty(XMPConst.NS_EXIF, "GPSLongitudeRef", gps_lng_ref);
				}
				else
				{
					// If we did not get coordinates, clear the exif fields;
					xmp.deleteProperty(XMPConst.NS_EXIF, "GPSVersionID");
					xmp.deleteProperty(XMPConst.NS_EXIF, "GPSLatitude");
					xmp.deleteProperty(XMPConst.NS_EXIF, "GPSLatitudeRef");
					xmp.deleteProperty(XMPConst.NS_EXIF, "GPSLongitude");			
					xmp.deleteProperty(XMPConst.NS_EXIF, "GPSLongitudeRef");
				}
			
				var xmpFile = new XMPFile(thumb.spec.fsName,XMPConst.FILE_UNKNOWN, XMPConst.OPEN_FOR_UPDATE);

				if (xmpFile.canPutXMP(xmp))
				{
				   xmpFile.putXMP(xmp);
				}
				xmpFile.closeFile(); 			
			
				// Write the packet back to the selected file
				var updatedPacket = xmp.serialize(XMPConst.SERIALIZE_OMIT_PACKET_WRAPPER | XMPConst.SERIALIZE_USE_COMPACT_FORMAT);

				// $.writeln(updatedPacket);
				thumb.metadata = new Metadata(updatedPacket);
				//app.beep();
			}
		}
	}




	for(var i = 0;i < app.documents.length;i++)
	{
		// Add the tabbed palette to all open Bridge windows
		addUI(app.documents[i]);
		
		// Update palette with current selection in all open Bridge windows
		//selectionsChanged(app.documents[i]);
	}

	// Create an event handler for the Document object
	this.onDocumentEvent = function( event )
	{
		if(event.object instanceof Document)
		{
			switch( event.type ) 
			{
				case "create":
					// When a new Bridge window is opened then add the tabbed palette
					addUI(event.object);
					break;
				case "destroy":
					// When a Bridge window is closed, remove the reference to it's tabbed palette
					removeUI(event.object);
					break;
				case "selectionsChanged":
					selectionsChanged(event.object);
					break;
			}
		}
		return { handled: false };  // Continue to do other handlers
	}
	
	// Add the event handler to the application
	app.eventHandlers.push( { handler: this.onDocumentEvent} );

	return this;
}

/**
  Ran before a new instance of BridgeGeotagger is created if a previous 
  instance exists. Cleans up traces of the previous instance:
  <ul>
  <li>Event handler
  <li>TabbedPalette
  </ul>
*/

BridgeGeotagger.prototype.destroy = function()
{
	// remove event handler
	for(var i = app.eventHandlers.length-1; i >= 0; i--) 
	{
		if (app.eventHandlers[i].handler == this.onDocumentEvent) 
		{
			app.eventHandlers.splice(i,1);
		}
	}

	for(var i = 0; i < app.documents.length; i++)
	{
		// Remove the TabbedPalette from all open documents
		var palettes = app.documents[i].palettes;
		for(var j = palettes.length-1; j >=0; j--) 
		{
				if(palettes[j].id == "geotaggingTab") 
				{
					palettes[j].remove();
				}
		}
	}
}

/**
  Determines whether snippet can be run given current context.  The snippet 
  fails if these preconditions are not met:
  <ul>
  <li> Must be running in Bridge
  <li> Google maps HTML file must exists
  </ul>
 
  @return True is this snippet can run, false otherwise
  @type boolean
*/
BridgeGeotagger.prototype.canRun = function()
{	
	// Need to be running in Bridge 
	if(BridgeTalk.appName == "bridge" && (this.htmlFile.exists)) 
	{
		return true;		
	}
	
	// Fail if these preconditions are not met.  
	// Bridge must be running,
	$.writeln("ERROR:: Cannot run BridgeGeotagger");
	$.writeln(this.requiredContext);
	return false;
	
}

/**
 "main program": construct an anonymous instance and run it
  as long as we are not unit-testing this snippet.
*/
var BridgeGeotaggerInstance
if(typeof(BridgeGeotagger_unitTest)  == "undefined") 
{
	if(BridgeGeotaggerInstance) 
	{
		BridgeGeotaggerInstance.destroy()
	}

	BridgeGeotaggerInstance = new BridgeGeotagger()
	BridgeGeotaggerInstance.run();
}

function gps2dec(gps_string, gps_string_ref) 
{
	// Convert from deg,min,sec representation to decimal value
	if (("NESW").indexOf( gps_string.substring(gps_string.length-1) ) >=0 ) 
	{
		gps_string_ref = gps_string.substring(gps_string.length-1);
		gps_string = gps_string.substring(0, gps_string.length-1);
	}

	var gps = gps_string.split(",");
	
	var deg = parseFloat(gps[0]);
	if ( gps_string_ref && ("SW").indexOf(gps_string_ref)>=0 )
		deg = -Math.abs(deg);
	var min = (gps.length >= 2) ? parseFloat(gps[1]) : 0;
	var sec = (gps.length >= 3) ? parseFloat(gps[2]) : 0;

	return deg + (min/60) + (sec/(60*60));
}

function dec2gps(dec_value) 
{
	// Convert from decimal value to deg,min,sec representation
	var deg = Math.floor(dec_value);
	var dec_min = 60 * (dec_value - deg);
	var min = Math.floor(dec_min);
	var sec = 60 * (dec_min - min);
	
	return [deg , min , sec];
}